#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import BANK
from holidays.countries.chile import Chile
from tests.common import CommonCountryTests


class TestChile(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Chile)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Chile(categories=BANK, years=range(self.start_year, 1956)))

    def test_special_holidays(self):
        self.assertHoliday(
            "1999-12-31",
            "2002-04-24",
            "2004-09-17",
            "2010-09-17",
            "2010-09-20",
            "2017-04-19",
            "2022-09-16",
        )

    def test_new_years_day(self):
        name_jan_2 = "Feriado nacional"
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))
        self.assertHolidayName(
            name_jan_2,
            "2017-01-02",
            "2023-01-02",
        )
        self.assertNoHolidayName(
            name_jan_2, (f"{year}-01-02" for year in range(self.start_year, 2017))
        )

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Sábado Santo"
        self.assertHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_ascension(self):
        name = "Ascensión del Señor"
        self.assertHolidayName(
            name,
            "1915-05-13",
            "1930-05-29",
            "1950-05-18",
            "1967-05-04",
        )
        self.assertHolidayName(name, range(self.start_year, 1968))
        self.assertNoHolidayName(name, range(1968, self.end_year))

    def test_corpus_christi(self):
        name = "Corpus Christi"
        self.assertHolidayName(
            name,
            "1915-06-03",
            "1940-05-23",
            "1950-06-08",
            "1967-05-25",
            "1987-06-18",
            "1998-06-11",
            "2000-06-19",
            "2006-06-12",
        )
        self.assertHolidayName(name, range(self.start_year, 1968), range(1987, 2007))
        self.assertNoHolidayName(name, range(1968, 1987), range(2007, self.end_year))

    def test_labor_day(self):
        name = "Día Nacional del Trabajo"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1932, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1932))

    def test_naval_glories_day(self):
        self.assertHolidayName(
            "Día de las Glorias Navales", (f"{year}-05-21" for year in self.full_range)
        )

    def test_indigenous_peoples_day(self):
        name = "Día Nacional de los Pueblos Indígenas"
        self.assertHolidayName(
            name,
            "2021-06-21",
            "2022-06-21",
            "2023-06-21",
            "2024-06-20",
            "2025-06-20",
            "2026-06-21",
            "2027-06-21",
            "2028-06-20",
            "2029-06-20",
            "2030-06-21",
            "2031-06-21",
            "2032-06-20",
            "2033-06-20",
            "2034-06-21",
            "2035-06-21",
            "2036-06-20",
            "2037-06-20",
            "2038-06-21",
            "2039-06-21",
            "2040-06-20",
            "2041-06-20",
            "2042-06-21",
            "2043-06-21",
            "2044-06-20",
            "2045-06-20",
            "2046-06-21",
            "2047-06-21",
            "2048-06-20",
            "2049-06-20",
        )
        self.assertNoHolidayName(name, range(self.start_year, 2021))

    def test_saint_peter_and_paul(self):
        name = "San Pedro y San Pablo"
        self.assertNonObservedHolidayName(
            name,
            (
                f"{year}-06-29"
                for year in (*range(self.start_year, 1968), *range(1986, self.end_year))
            ),
        )
        self.assertHolidayName(
            name,
            "2000-06-26",
            "2001-07-02",
            "2002-06-29",
            "2003-06-29",
            "2004-06-28",
            "2005-06-27",
            "2006-06-26",
            "2007-07-02",
            "2008-06-29",
            "2009-06-29",
            "2010-06-28",
            "2011-06-27",
            "2012-07-02",
            "2013-06-29",
            "2014-06-29",
            "2015-06-29",
            "2016-06-27",
            "2017-06-26",
            "2018-07-02",
            "2019-06-29",
            "2020-06-29",
            "2021-06-28",
            "2022-06-27",
            "2023-06-26",
            "2024-06-29",
        )
        self.assertNoHolidayName(name, range(1968, 1986))

    def test_virgin_of_carmen(self):
        name = "Virgen del Carmen"
        self.assertHolidayName(name, (f"{year}-07-16" for year in range(2007, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2007))

    def test_assumption_of_mary(self):
        self.assertHolidayName(
            "Asunción de la Virgen", (f"{year}-08-15" for year in self.full_range)
        )

    def test_day_of_national_liberation(self):
        name = "Día de la Liberación Nacional"
        self.assertHolidayName(name, (f"{year}-09-11" for year in range(1981, 1999)))
        self.assertNoHolidayName(name, range(self.start_year, 1981), range(1999, self.end_year))

    def test_day_of_national_unity(self):
        name = "Día de la Unidad Nacional"
        self.assertHolidayName(
            name,
            "1999-09-06",
            "2000-09-04",
            "2001-09-03",
        )
        self.assertNoHolidayName(name, range(self.start_year, 1999), range(2002, self.end_year))

    def test_national_holidays(self):
        name = "Fiestas Patrias"
        self.assertHolidayName(
            name,
            "2007-09-17",
            "2012-09-17",
            "2013-09-20",
            "2018-09-17",
            "2019-09-20",
            "2021-09-17",
            "2024-09-20",
        )
        self.assertHolidayName(name, (f"{year}-09-20" for year in range(1932, 1945)))
        self.assertNoHolidayName(name, range(self.start_year, 1932), range(1945, 2007))

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de la Independencia", (f"{year}-09-18" for year in self.full_range)
        )

    def test_army_day(self):
        self.assertHolidayName(
            "Día de las Glorias del Ejército", (f"{year}-09-19" for year in self.full_range)
        )

    def test_columbus_day(self):
        name_1922 = "Día de la Raza"
        name_2000 = "Día del Encuentro de dos Mundos"
        self.assertHolidayName(
            name_1922, (f"{year}-10-12" for year in (*range(1922, 1973), *range(1974, 2000)))
        )
        self.assertNoHolidayName(name_1922, 1973, range(2000, self.end_year))

        self.assertHolidayName(
            name_2000,
            "2000-10-09",
            "2005-10-10",
            "2010-10-11",
            "2015-10-12",
            "2019-10-12",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
        )
        self.assertHolidayName(name_2000, range(2000, self.end_year))

    def test_national_day_of_evangelical_and_protestant_churches(self):
        name = "Día Nacional de las Iglesias Evangélicas y Protestantes"
        self.assertHolidayName(
            name,
            "2008-10-31",
            "2009-10-31",
            "2010-10-31",
            "2011-10-31",
            "2012-11-02",
            "2013-10-31",
            "2014-10-31",
            "2015-10-31",
            "2016-10-31",
            "2017-10-27",
            "2018-11-02",
            "2019-10-31",
            "2020-10-31",
            "2021-10-31",
            "2022-10-31",
            "2023-10-27",
        )
        self.assertNoHolidayName(name, range(self.start_year, 2008))

    def test_all_saints_day(self):
        self.assertHolidayName(
            "Día de Todos los Santos", (f"{year}-11-01" for year in self.full_range)
        )

    def test_immaculate_conception(self):
        self.assertHolidayName(
            "La Inmaculada Concepción", (f"{year}-12-08" for year in self.full_range)
        )

    def test_christmas_eve(self):
        name = "Víspera de Navidad"
        self.assertHolidayName(name, (f"{year}-12-24" for year in range(1944, 1989)))
        self.assertNoHolidayName(name, range(self.start_year, 1944), range(1989, self.end_year))

    def test_christmas_day(self):
        self.assertHolidayName("Navidad", (f"{year}-12-25" for year in self.full_range))

    def test_bank_holidays(self):
        name = "Feriado bancario"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-06-30" for year in range(1957, 1976)))
        self.assertBankHolidayName(
            name, (f"{year}-12-31" for year in (*range(1956, 1997), *range(1998, self.end_year)))
        )
        self.assertNoBankHolidayName(name, range(self.start_year, 1956), 1997)

    def test_2019(self):
        self.assertHolidayDatesInYear(
            2019,
            "2019-01-01",
            "2019-04-19",
            "2019-04-20",
            "2019-05-01",
            "2019-05-21",
            "2019-06-29",
            "2019-07-16",
            "2019-08-15",
            "2019-09-18",
            "2019-09-19",
            "2019-09-20",
            "2019-10-12",
            "2019-10-31",
            "2019-11-01",
            "2019-12-08",
            "2019-12-25",
        )

    def test_2020(self):
        # from https://feriados.cl/2020.htm
        self.assertHolidayDatesInYear(
            2020,
            "2020-01-01",
            "2020-04-10",
            "2020-04-11",
            "2020-05-01",
            "2020-05-21",
            "2020-06-29",
            "2020-07-16",
            "2020-08-15",
            "2020-09-18",
            "2020-09-19",
            "2020-10-12",
            "2020-10-31",
            "2020-11-01",
            "2020-12-08",
            "2020-12-25",
        )

    def test_2021(self):
        # from https://feriados.cl/2021.htm
        self.assertHolidayDatesInYear(
            2021,
            "2021-01-01",
            "2021-04-02",
            "2021-04-03",
            "2021-05-01",
            "2021-05-21",
            "2021-06-21",
            "2021-06-28",
            "2021-07-16",
            "2021-08-15",
            "2021-09-17",
            "2021-09-18",
            "2021-09-19",
            "2021-10-11",
            "2021-10-31",
            "2021-11-01",
            "2021-12-08",
            "2021-12-25",
        )

    def test_assault_and_capture_of_cape_arica(self):
        name = "Asalto y Toma del Morro de Arica"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AP":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-07" for year in range(2013, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2013))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_nativity_of_bernado_o_higgins_chillan_and_chillan_viejo_commune(self):
        name = "Nacimiento del Prócer de la Independencia (Chillán y Chillán Viejo)"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NB":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-20" for year in range(2014, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2014))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-04-16", "Sábado Santo"),
            ("2022-05-01", "Día Nacional del Trabajo"),
            ("2022-05-21", "Día de las Glorias Navales"),
            ("2022-06-07", "Asalto y Toma del Morro de Arica"),
            ("2022-06-21", "Día Nacional de los Pueblos Indígenas"),
            ("2022-06-27", "San Pedro y San Pablo"),
            ("2022-07-16", "Virgen del Carmen"),
            ("2022-08-15", "Asunción de la Virgen"),
            ("2022-08-20", "Nacimiento del Prócer de la Independencia (Chillán y Chillán Viejo)"),
            ("2022-09-16", "Feriado nacional"),
            ("2022-09-18", "Día de la Independencia"),
            ("2022-09-19", "Día de las Glorias del Ejército"),
            ("2022-10-10", "Día del Encuentro de dos Mundos"),
            ("2022-10-31", "Día Nacional de las Iglesias Evangélicas y Protestantes"),
            ("2022-11-01", "Día de Todos los Santos"),
            ("2022-12-08", "La Inmaculada Concepción"),
            ("2022-12-25", "Navidad"),
            ("2022-12-31", "Feriado bancario"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Holy Saturday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-21", "Navy Day"),
            ("2022-06-07", "Assault and Capture of Cape Arica"),
            ("2022-06-21", "National Day of Indigenous Peoples"),
            ("2022-06-27", "Saint Peter and Saint Paul's Day"),
            ("2022-07-16", "Our Lady of Mount Carmel"),
            ("2022-08-15", "Assumption Day"),
            ("2022-08-20", "Nativity of Bernardo O'Higgins (Chillán and Chillán Viejo communes)"),
            ("2022-09-16", "National Holiday"),
            ("2022-09-18", "Independence Day"),
            ("2022-09-19", "Army Day"),
            ("2022-10-10", "Meeting of Two Worlds' Day"),
            ("2022-10-31", "Reformation Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-08", "Immaculate Conception"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-31", "Bank Holiday"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-16", "Велика субота"),
            ("2022-05-01", "День праці"),
            ("2022-05-21", "День військово-морської слави"),
            ("2022-06-07", "Штурм і захоплення Морро-де-Аріка"),
            ("2022-06-21", "Національний день корінних народів"),
            ("2022-06-27", "День Святих Петра і Павла"),
            ("2022-07-16", "Матір Божа Кармельська"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-08-20", "Річниця Бернардо ОʼГіґґінса (свято комун Чіллан і Чіллан Вʼєхо)"),
            ("2022-09-16", "Національне свято"),
            ("2022-09-18", "День незалежності"),
            ("2022-09-19", "День військової слави"),
            ("2022-10-10", "День зустрічі двох світів"),
            ("2022-10-31", "День Реформації"),
            ("2022-11-01", "День усіх святих"),
            ("2022-12-08", "Непорочне зачаття Діви Марії"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-31", "Банківський вихідний"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.christmas_island import ChristmasIsland
from tests.common import CommonCountryTests


class TestChristmasIsland(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(ChristmasIsland)

    def test_special_holidays(self):
        self.assertHoliday("2022-09-22")

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_chinese_new_year(self):
        name = "Chinese New Year"
        self.assertHolidayName(
            name,
            "2021-02-12",
            "2021-02-13",
            "2022-02-01",
            "2022-02-02",
            "2023-01-23",
            "2023-01-24",
            "2024-02-10",
            "2024-02-11",
            "2025-01-29",
            "2025-01-30",
        )
        self.assertHolidayName(name, range(self.start_year, 2020), range(2021, self.end_year))
        self.assertNoHolidayName(name, 2020)
        obs_dts = (
            "2018-02-19",
            "2020-01-28",
            "2020-01-29",
            "2021-02-15",
            "2024-02-12",
            "2024-02-13",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_australia_day(self):
        name = "Australia Day"
        self.assertHolidayName(name, (f"{year}-01-26" for year in self.full_range))
        obs_dts = (
            "2013-01-28",
            "2014-01-27",
            "2019-01-28",
            "2020-01-27",
            "2025-01-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_labour_day(self):
        name = "Labour Day"
        self.assertHolidayName(
            name,
            "2020-03-16",
            "2021-03-22",
            "2022-03-21",
            "2023-03-20",
            "2024-03-18",
            "2025-03-24",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_anzac_day(self):
        name = "ANZAC Day"
        self.assertHolidayName(name, (f"{year}-04-25" for year in self.full_range))
        obs_dts = (
            "2009-04-27",
            "2010-04-26",
            "2015-04-27",
            "2020-04-27",
            "2021-04-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_territory_day(self):
        name = "Territory Day"
        self.assertHolidayName(
            name,
            "2013-10-07",
            "2018-10-01",
            "2019-10-07",
            "2020-10-05",
            "2024-10-07",
        )
        self.assertHolidayName(name, self.full_range)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-26",
            "2016-12-26",
            "2021-12-27",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2015-12-28",
            "2016-12-27",
            "2020-12-28",
            "2021-12-28",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Hari Raya Puasa"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-22",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2012-08-20",
            "2017-06-26",
            "2020-05-25",
            "2023-04-24",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Hari Raya Haji"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-17",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2011-11-07",
            "2014-10-07",
            "2019-08-12",
            "2022-07-11",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-01-26", "Australia Day"),
            ("2024-02-10", "Chinese New Year"),
            ("2024-02-11", "Chinese New Year"),
            ("2024-02-12", "Chinese New Year (observed)"),
            ("2024-02-13", "Chinese New Year (observed)"),
            ("2024-03-18", "Labour Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-10", "Hari Raya Puasa"),
            ("2024-04-25", "ANZAC Day"),
            ("2024-06-17", "Hari Raya Haji"),
            ("2024-10-07", "Territory Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_en_cx(self):
        self.assertLocalizedHolidays(
            "en_CX",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-26", "Australia Day"),
            ("2024-02-10", "Chinese New Year"),
            ("2024-02-11", "Chinese New Year"),
            ("2024-02-12", "Chinese New Year (observed)"),
            ("2024-02-13", "Chinese New Year (observed)"),
            ("2024-03-18", "Labour Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-10", "Hari Raya Puasa"),
            ("2024-04-25", "ANZAC Day"),
            ("2024-06-17", "Hari Raya Haji"),
            ("2024-10-07", "Territory Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-26", "Australia Day"),
            ("2024-02-10", "Chinese New Year"),
            ("2024-02-11", "Chinese New Year"),
            ("2024-02-12", "Chinese New Year (observed)"),
            ("2024-02-13", "Chinese New Year (observed)"),
            ("2024-03-18", "Labor Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-04-25", "ANZAC Day"),
            ("2024-06-17", "Eid al-Adha"),
            ("2024-10-07", "Territory Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import SUN
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_PREV_SAT


class DRCongo(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Democratic Republic of the Congo holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Democratic_Republic_of_the_Congo>
        * [Ordonnance n° 79-154](https://web.archive.org/web/20220329181351/http://www.leganet.cd/Legislation/DroitSocial/O.79.154.23.06.1979.htm)
        * [Ordonnance n° 14/010](https://web.archive.org/web/20230419184344/http://leganet.cd/Legislation/Divers/Ordonnance.14.10.14.mai.2014.htm)
        * [Ordonnance n° 23-042](https://web.archive.org/web/20250113230411/http://www.droitcongolais.info/files/143.03.23_Ordonnance-du-30-mars-2023_jours-feries.pdf)
        * [Loi n° 009-2002](https://web.archive.org/web/20250104233847/https://www.leganet.cd/Legislation/Droit%20administratif/Urbanismevoiries/Div/L.009.05.08.2002.htm)
    """

    country = "CD"
    default_language = "fr"
    # %s (observed).
    observed_label = tr("%s (observé)")
    supported_languages = ("en_US", "fr")
    # Ordonnance n° 79-154.
    start_year = 1980
    weekend = {SUN}

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SUN_TO_PREV_SAT)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        name = tr("Nouvel an")
        self._add_new_years_day(name)
        self._add_observed(self._next_year_new_years_day, name=name)

        # Established on May 10th, 2014 via Ordonnance n° 14/010.
        if self._year >= 2015:
            # Martyrs' Day.
            self._add_observed(self._add_holiday_jan_4(tr("Martyrs de l'indépendance")))

            self._add_observed(
                # National Hero Laurent Désiré Kabila Day.
                self._add_holiday_jan_16(tr("Journée du héros national Laurent Désiré Kabila"))
            )

            self._add_observed(
                # National Hero Patrice Emery Lumumba Day.
                self._add_holiday_jan_17(tr("Journée du héros national Patrice Emery Lumumba"))
            )

        # Established on March 30th, 2023 via Ordonnance n° 23-042.
        if self._year >= 2023:
            self._add_observed(
                self._add_holiday_apr_6(
                    # Day of the Struggle of Simon Kimbangu and African Consciousness.
                    tr("Journée du combat de Simon Kimbangu et de la conscience africaine")
                )
            )

        # Labor Day.
        self._add_observed(self._add_labor_day(tr("Fête du travail")))

        # Renamed on May 10th, 2014 via Ordonnance n° 14/010.
        if self._year >= 2014:
            self._add_observed(
                # Revolution and Armed Forces Day.
                self._add_holiday_may_17(tr("Journée de la Révolution et des Forces Armées"))
            )
        else:
            # Armed Forces Day.
            self._add_observed(self._add_holiday_nov_17(tr("Fête des Forces armées zaïroises")))

        # Removed on May 10th, 2014 via Ordonnance n° 14/010.
        if self._year <= 2013:
            self._add_observed(
                self._add_holiday_may_20(
                    # Anniversary of the Popular Movement of the Revolution.
                    tr("Anniversaire du Mouvement populaire de la révolution")
                )
            )

            self._add_observed(
                self._add_holiday_jun_24(
                    # Anniversary of the New Revolutionary Constitution.
                    tr("Anniversaire de la nouvelle Constitution révolutionnaire")
                )
            )

        # Independence Day.
        self._add_observed(self._add_holiday_jun_30(tr("Journée de l'indépendance")))

        # Parents' Day.
        self._add_observed(self._add_holiday_aug_1(tr("Fête des parents")))

        if self._year >= 2024:
            self._add_observed(
                # Congolese Genocide Memorial Day.
                self._add_holiday_aug_2(tr("Journée commémorative du génocide Congolais"))
            )

        # Removed on May 10th, 2014 via Ordonnance n° 14/010.
        if self._year <= 2013:
            # Youth Day.
            self._add_observed(self._add_holiday_oct_14(tr("Journée de la Jeunesse")))

            self._add_observed(
                # Anniversary of the Country's Name Change.
                self._add_holiday_oct_27(tr("Anniversaire du changement du nom de notre Pays"))
            )

            # Anniversary of the New Regime.
            self._add_observed(self._add_holiday_nov_24(tr("Anniversaire du nouveau régime")))

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Noël")))


class CD(DRCongo):
    pass


class COD(DRCongo):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate reference pages for countries and markets inside docs/auto_gen_docs."""

from pathlib import Path

import mkdocs_gen_files


def main():
    root = Path(__file__).parents[2]  # project folder
    holidays_path = root / "holidays"

    for entity_type in ("countries", "financial"):
        for path in sorted((holidays_path / entity_type).rglob("*.py")):
            file_name = path.stem
            if file_name == "__init__":
                continue
            doc_path = Path("auto_gen_docs", f"{file_name}.md")

            with mkdocs_gen_files.open(doc_path, "w") as f:
                f.write(f"::: holidays.{entity_type}.{file_name}")
            mkdocs_gen_files.set_edit_path(doc_path, path.relative_to(root))


if __name__ in {"__main__", "<run_path>"}:
    main()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, JUL, AUG, SEP, OCT, NOV, DEC, THU, FRI, SAT
from holidays.constants import BANK, PUBLIC
from holidays.groups import InternationalHolidays, IslamicHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Qatar(HolidayBase, InternationalHolidays, IslamicHolidays, StaticHolidays):
    """Qatar holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Qatar>
        * [National Sports Day](https://web.archive.org/web/20250417141518/https://hukoomi.gov.qa/en/national-sport-day)
        * [Qatar National Day](https://web.archive.org/web/20240522081644/https://www.qatar.qa/en/qatar/history-of-qatar-qatar-national-day-committee/)
        * [Weekend](https://web.archive.org/web/20240930093123/https://www.arabnews.com/node/234601)
    """

    country = "QA"
    default_language = "ar_QA"
    # %s (estimated).
    estimated_label = tr("%s (تقديري)")
    start_year = 1971
    supported_categories = (BANK, PUBLIC)
    supported_languages = ("ar_QA", "en_US")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=QatarIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, QatarStaticHolidays)
        super().__init__(*args, **kwargs)

    def _get_weekend(self, dt: date) -> set[int]:
        # Qatar switches from THU-FRI to FRI-SAT on Aug 1, 2003.
        return {FRI, SAT} if dt >= date(2003, AUG, 1) else {THU, FRI}

    def _populate_public_holidays(self):
        if self._year >= 2012:
            # National Sports Day.
            self._add_holiday_2nd_tue_of_feb(tr("اليوم الوطني للرياضة"))

        if self._year >= 2007:
            # Qatar National Day.
            self._add_holiday_dec_18(tr("اليوم الوطني لقطر"))

        # Eid al-Fitr.
        name = tr("عيد الفطر")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)

        # Eid al-Adha.
        name = tr("عيد الأضحى")
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)

    def _populate_bank_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("رأس السنة الميلادية"))

        if self._year >= 2010:
            # March Bank Holiday.
            self._add_holiday_1st_sun_of_mar(tr("عطلة البنك"))


class QA(Qatar):
    pass


class QAT(Qatar):
    pass


class QatarIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20260107122007/https://www.timeanddate.com/holidays/qatar/eid-al-adha
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2005, 2025)
    EID_AL_ADHA_DATES = {
        2008: (DEC, 9),
        2009: (NOV, 28),
        2010: (NOV, 15),
        2016: (SEP, 10),
        2017: (AUG, 31),
        2018: (AUG, 22),
    }

    # https://web.archive.org/web/20241207022523/https://www.timeanddate.com/holidays/qatar/eid-al-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2005, 2025)
    EID_AL_FITR_DATES = {
        2005: (NOV, 4),
        2006: (OCT, 24),
        2008: (OCT, 2),
        2009: (SEP, 21),
        2011: (AUG, 31),
        2015: (JUL, 18),
    }


class QatarStaticHolidays:
    """Qatar special holidays.

    References:
        * [New Year's Holiday](https://web.archive.org/web/20250119073018/https://www.expatica.com/qa/lifestyle/holidays/qatar-public-holidays-74585/)
    """

    special_public_holidays = {
        # New Year's Holiday.
        2025: (JAN, 2, tr("عطلة رأس السنة")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.greenland import Greenland
from tests.common import CommonCountryTests


class TestGreenland(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Greenland)

    def test_1982(self):
        self.assertHolidaysInYear(
            1982,
            ("1982-01-01", "Ukiortaaq"),
            ("1982-04-08", "Sisamanngortoq illernartoq"),
            ("1982-04-09", "Tallimanngorneq tannaartoq"),
            ("1982-04-11", "Poorskip ullua"),
            ("1982-04-12", "Poorskip-aappaa"),
            ("1982-05-07", "Ulloq qinuffiusoq"),
            ("1982-05-20", "Qilaliarfik"),
            ("1982-05-30", "Piinsip ullua"),
            ("1982-05-31", "Piinsip-aappaa"),
            ("1982-12-25", "Juullip ullua"),
            ("1982-12-26", "Juullip-aappaa"),
        )

    def test_1982_optional(self):
        self.assertOptionalHolidaysInYear(
            1982,
            ("1982-01-06", "Kunngit pingasut ulluat"),
            ("1982-05-01", "Sulisartut ulluat"),
            ("1982-12-24", "Juulliaraq"),
            ("1982-12-31", "Ukiortaami"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Ukiortaaq"),
            ("2022-04-14", "Sisamanngortoq illernartoq"),
            ("2022-04-15", "Tallimanngorneq tannaartoq"),
            ("2022-04-17", "Poorskip ullua"),
            ("2022-04-18", "Poorskip-aappaa"),
            ("2022-05-13", "Ulloq qinuffiusoq"),
            ("2022-05-26", "Qilaliarfik"),
            ("2022-06-05", "Piinsip ullua"),
            ("2022-06-06", "Piinsip-aappaa"),
            ("2022-12-25", "Juullip ullua"),
            ("2022-12-26", "Juullip-aappaa"),
        )

    def test_2022_optional(self):
        self.assertOptionalHolidaysInYear(
            2022,
            ("2022-01-06", "Kunngit pingasut ulluat"),
            ("2022-05-01", "Sulisartut ulluat"),
            ("2022-06-21", "Ullortuneq"),
            ("2022-12-24", "Juulliaraq"),
            ("2022-12-31", "Ukiortaami"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Ukiortaaq"),
            ("2022-01-06", "Kunngit pingasut ulluat"),
            ("2022-04-14", "Sisamanngortoq illernartoq"),
            ("2022-04-15", "Tallimanngorneq tannaartoq"),
            ("2022-04-17", "Poorskip ullua"),
            ("2022-04-18", "Poorskip-aappaa"),
            ("2022-05-01", "Sulisartut ulluat"),
            ("2022-05-13", "Ulloq qinuffiusoq"),
            ("2022-05-26", "Qilaliarfik"),
            ("2022-06-05", "Piinsip ullua"),
            ("2022-06-06", "Piinsip-aappaa"),
            ("2022-06-21", "Ullortuneq"),
            ("2022-12-24", "Juulliaraq"),
            ("2022-12-25", "Juullip ullua"),
            ("2022-12-26", "Juullip-aappaa"),
            ("2022-12-31", "Ukiortaami"),
        )

    def test_l10n_da(self):
        self.assertLocalizedHolidays(
            "da",
            ("2022-01-01", "Nytårsdag"),
            ("2022-01-06", "Helligtrekongersdag"),
            ("2022-04-14", "Skærtorsdag"),
            ("2022-04-15", "Langfredag"),
            ("2022-04-17", "Påskedag"),
            ("2022-04-18", "Anden påskedag"),
            ("2022-05-01", "Arbejdernes kampdag"),
            ("2022-05-13", "Store bededag"),
            ("2022-05-26", "Kristi himmelfartsdag"),
            ("2022-06-05", "Pinsedag"),
            ("2022-06-06", "Anden pinsedag"),
            ("2022-06-21", "Nationaldag"),
            ("2022-12-24", "Juleaftensdag"),
            ("2022-12-25", "Juledag"),
            ("2022-12-26", "Anden juledag"),
            ("2022-12-31", "Nytårsaften"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "International Workers' Day"),
            ("2022-05-13", "Great Prayer Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-21", "National Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_fi(self):
        self.assertLocalizedHolidays(
            "fi",
            ("2022-01-01", "Uudenvuodenpäivä"),
            ("2022-01-06", "Loppiainen"),
            ("2022-04-14", "Kiirastorstai"),
            ("2022-04-15", "Pitkäperjantai"),
            ("2022-04-17", "Pääsiäispäivä"),
            ("2022-04-18", "Toinen pääsiäispäivä"),
            ("2022-05-01", "Kansainvälinen työn päivä"),
            ("2022-05-13", "Suuri rukouspäivä"),
            ("2022-05-26", "Helatorstai"),
            ("2022-06-05", "Helluntaipäivä"),
            ("2022-06-06", "Toinen helluntaipäivä"),
            ("2022-06-21", "Kansallispäivä"),
            ("2022-12-24", "Jouluaatto"),
            ("2022-12-25", "Joulupäivä"),
            ("2022-12-26", "Tapaninpäivä"),
            ("2022-12-31", "Uudenvuodenaatto"),
        )

    def test_l10n_is(self):
        self.assertLocalizedHolidays(
            "is",
            ("2022-01-01", "Nýársdagur"),
            ("2022-01-06", "Þrettándinn"),
            ("2022-04-14", "Skírdagur"),
            ("2022-04-15", "Föstudagurinn langi"),
            ("2022-04-17", "Páskadagur"),
            ("2022-04-18", "Annar í páskum"),
            ("2022-05-01", "Frídagur verkalýðsins"),
            ("2022-05-13", "Kóngsbænadagur"),
            ("2022-05-26", "Uppstigningardagur"),
            ("2022-06-05", "Hvítasunnudagur"),
            ("2022-06-06", "Annar í hvítasunnu"),
            ("2022-06-21", "Þjóðhátíðardagur"),
            ("2022-12-24", "Aðfangadagur"),
            ("2022-12-25", "Jóladagur"),
            ("2022-12-26", "Annar í jólum"),
            ("2022-12-31", "Gamlársdagur"),
        )

    def test_l10n_no(self):
        self.assertLocalizedHolidays(
            "no",
            ("2022-01-01", "Nyttårsdag"),
            ("2022-01-06", "Helligtrekongersdag"),
            ("2022-04-14", "Skjærtorsdag"),
            ("2022-04-15", "Langfredag"),
            ("2022-04-17", "Første påskedag"),
            ("2022-04-18", "Andre påskedag"),
            ("2022-05-01", "Arbeidernes internasjonale kampdag"),
            ("2022-05-13", "Store bededag"),
            ("2022-05-26", "Kristi himmelfartsdag"),
            ("2022-06-05", "Første pinsedag"),
            ("2022-06-06", "Andre pinsedag"),
            ("2022-06-21", "Nasjonaldag"),
            ("2022-12-24", "Julaften"),
            ("2022-12-25", "Juledag"),
            ("2022-12-26", "Andre juledag"),
            ("2022-12-31", "Nyttårsaften"),
        )

    def test_l10n_sv(self):
        self.assertLocalizedHolidays(
            "sv",
            ("2022-01-01", "Nyårsdagen"),
            ("2022-01-06", "Trettondedag jul"),
            ("2022-04-14", "Skärtorsdagen"),
            ("2022-04-15", "Långfredagen"),
            ("2022-04-17", "Påskdagen"),
            ("2022-04-18", "Annandag påsk"),
            ("2022-05-01", "Internationella arbetardagen"),
            ("2022-05-13", "Stora bönedagen"),
            ("2022-05-26", "Kristi himmelsfärdsdag"),
            ("2022-06-05", "Pingstdagen"),
            ("2022-06-06", "Annandag pingst"),
            ("2022-06-21", "Nationaldag"),
            ("2022-12-24", "Julafton"),
            ("2022-12-25", "Juldagen"),
            ("2022-12-26", "Annandag jul"),
            ("2022-12-31", "Nyårsafton"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-06", "Богоявлення"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День трудящих"),
            ("2022-05-13", "День загальної молитви"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-06-21", "Національне свято"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
            ("2022-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.gabon import Gabon
from tests.common import CommonCountryTests


class TestGabon(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Gabon)

    def test_special_holidays(self):
        self.assertHoliday(
            "2024-11-14",
            "2024-11-15",
            "2024-11-16",
            "2025-04-11",
            "2025-04-12",
            "2025-05-02",
            "2025-09-27",
        )

    def test_new_years_day(self):
        self.assertHolidayName("Jour de l'an", (f"{year}-01-01" for year in self.full_range))

    def test_renovation_day(self):
        name = "Journée de la rénovation"
        self.assertHolidayName(name, (f"{year}-03-12" for year in range(self.start_year, 1993)))
        self.assertNoHolidayName(name, range(1993, self.end_year))

    def test_easter_sunday(self):
        name = "Pâques"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        self.assertHolidayName(
            name,
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_womens_rights_day(self):
        name = "Journée des droits de la femme"
        self.assertHolidayName(name, (f"{year}-04-17" for year in range(2015, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2015))

    def test_labor_day(self):
        self.assertHolidayName("Fête du Travail", (f"{year}-05-01" for year in self.full_range))

    def test_youth_day(self):
        name = "Fête de la Jeunesse"
        self.assertHolidayName(
            name,
            "1987-05-10",
            "1988-05-08",
            "1989-05-14",
            "1990-05-13",
            "1991-05-12",
        )
        self.assertHolidayName(name, range(self.start_year, 1992))
        self.assertNoHolidayName(name, range(1992, self.end_year))

    def test_ascension_day(self):
        name = "Ascension"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, range(2018, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2018))

    def test_whit_sunday(self):
        name = "Pentecôte"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Lundi de Pentecôte"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_assumption_day(self):
        self.assertHolidayName(
            "Assomption de Marie", (f"{year}-08-15" for year in self.full_range)
        )

    def test_independence_day(self):
        self.assertHolidayName(
            "Jour de l'indépendance", (f"{year}-08-16" for year in self.full_range)
        )

    def test_independence_day_holiday(self):
        name = "Fête de l'indépendance"
        self.assertHolidayName(name, (f"{year}-08-17" for year in self.full_range))
        self.assertHolidayName(name, (f"{year}-08-18" for year in range(self.start_year, 1992)))
        self.assertHolidayNameCount(name, 1, range(1992, self.end_year))

    def test_liberation_day(self):
        name = "Journée de la Libération"
        self.assertHolidayName(name, (f"{year}-08-30" for year in range(2024, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2024))

    def test_all_saints_day(self):
        self.assertHolidayName("Toussaint", (f"{year}-11-01" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Noël", (f"{year}-12-25" for year in self.full_range))

    def test_eid_al_fitr(self):
        name = "Fin du Ramadan"
        self.assertHolidayName(
            name,
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Fête du sacrifice"
        self.assertHolidayName(
            name,
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Jour de l'an"),
            ("2024-03-31", "Pâques"),
            ("2024-04-01", "Lundi de Pâques"),
            ("2024-04-10", "Fin du Ramadan"),
            ("2024-04-17", "Journée des droits de la femme"),
            ("2024-05-01", "Fête du Travail"),
            ("2024-05-09", "Ascension"),
            ("2024-05-19", "Pentecôte"),
            ("2024-05-20", "Lundi de Pentecôte"),
            ("2024-06-16", "Fête du sacrifice"),
            ("2024-08-15", "Assomption de Marie"),
            ("2024-08-16", "Jour de l'indépendance"),
            ("2024-08-17", "Fête de l'indépendance"),
            ("2024-08-30", "Journée de la Libération"),
            ("2024-11-01", "Toussaint"),
            ("2024-11-14", "Jour férié"),
            ("2024-11-15", "Jour férié"),
            ("2024-11-16", "Jour férié"),
            ("2024-12-25", "Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-31", "Easter Sunday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-04-17", "Women's Rights Day"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-09", "Ascension Day"),
            ("2024-05-19", "Whit Sunday"),
            ("2024-05-20", "Whit Monday"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-08-15", "Assumption Day"),
            ("2024-08-16", "Independence Day"),
            ("2024-08-17", "Independence Day Holiday"),
            ("2024-08-30", "Liberation Day"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-11-14", "Public holiday"),
            ("2024-11-15", "Public holiday"),
            ("2024-11-16", "Public holiday"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.ethiopia import Ethiopia
from tests.common import CommonCountryTests


class TestEthiopia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Ethiopia)

    def test_christmas_day(self):
        self.assertHolidayName("የገና ወይም የልደት በዓል", (f"{year}-01-07" for year in self.full_range))

    def test_ephiphany(self):
        self.assertHolidayName(
            "የጥምቀት በዓል",
            (f"{year}-01-19" for year in self.full_range if year % 4 != 0),
            (f"{year}-01-20" for year in self.full_range if year % 4 == 0),
        )

    def test_adwa_victory_day(self):
        name = "የአድዋ ድል በዓል"
        self.assertHolidayName(name, (f"{year}-03-02" for year in range(1996, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1996))

    def test_good_friday(self):
        name = "የስቅለት በዓል"
        self.assertHolidayName(
            name,
            "2018-04-06",
            "2019-04-26",
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "የትንሳኤ(ፋሲካ) በዓል"
        self.assertHolidayName(
            name,
            "2018-04-08",
            "2019-04-28",
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_international_workers_day(self):
        self.assertHolidayName(
            "የዓለም የሠራተኞች (የላብአደሮች) ቀን", (f"{year}-05-01" for year in self.full_range)
        )

    def test_ethiopian_patriots_victory_day(self):
        self.assertHolidayName(
            "የአርበኞች (የድል) ቀን በዓል", (f"{year}-05-05" for year in self.full_range)
        )

    def test_downfall_of_the_dergue_regime_day(self):
        name = "ደርግ የወደቀበት ቀን"
        self.assertHolidayName(name, (f"{year}-05-28" for year in range(1992, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1992))

    def test_ethiopian_new_year(self):
        self.assertHolidayName(
            "የዘመን መለወጫ (እንቁጣጣሽ) በዓል",
            (f"{year}-09-11" for year in self.full_range if year % 4 != 3),
            (f"{year}-09-12" for year in self.full_range if year % 4 == 3),
        )

    def test_finding_of_true_cross(self):
        self.assertHolidayName(
            "የመስቀል በዓል",
            (f"{year}-09-27" for year in self.full_range if year % 4 != 3),
            (f"{year}-09-28" for year in self.full_range if year % 4 == 3),
        )

    def test_popular_revolution_commemoration_day(self):
        name = "የአብዮት ቀን"
        self.assertHolidayName(
            name,
            (f"{year}-09-12" for year in range(self.start_year, 1991) if year % 4 != 3),
            (f"{year}-09-13" for year in range(self.start_year, 1991) if year % 4 == 3),
        )
        self.assertNoHolidayName(name, range(1991, self.end_year))

    def test_october_revolution_day(self):
        name = "የጥቅምት አብዮት ቀን"
        self.assertHolidayName(name, (f"{year}-11-07" for year in range(self.start_year, 1991)))
        self.assertNoHolidayName(name, range(1991, self.end_year))

    def test_eid_al_fitr(self):
        name = "የኢድ አልፈጥር"
        self.assertHolidayName(
            name,
            "2018-06-15",
            "2019-06-04",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "የኢድ አልአድሃ (አረፋ)"
        self.assertHolidayName(
            name,
            "2018-08-22",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophets_birthday(self):
        name = "የመውሊድ በዓል"
        self.assertHolidayName(
            name,
            "2018-11-21",
            "2019-11-10",
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_ethiopian_martyrs_day(self):
        name = "የሰማዕታት ቀን"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-02-20" for year in self.full_range))

    def test_nations_nationality_and_peoples_day(self):
        name = "የብሔር ብሔረሰቦች ቀን"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-12-09" for year in range(2006, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2006))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-07", "የገና ወይም የልደት በዓል"),
            ("2022-01-19", "የጥምቀት በዓል"),
            ("2022-02-20", "የሰማዕታት ቀን"),
            ("2022-03-02", "የአድዋ ድል በዓል"),
            ("2022-04-22", "የስቅለት በዓል"),
            ("2022-04-24", "የትንሳኤ(ፋሲካ) በዓል"),
            ("2022-05-01", "የዓለም የሠራተኞች (የላብአደሮች) ቀን"),
            ("2022-05-02", "የኢድ አልፈጥር"),
            ("2022-05-05", "የአርበኞች (የድል) ቀን በዓል"),
            ("2022-05-28", "ደርግ የወደቀበት ቀን"),
            ("2022-07-09", "የኢድ አልአድሃ (አረፋ)"),
            ("2022-09-11", "የዘመን መለወጫ (እንቁጣጣሽ) በዓል"),
            ("2022-09-27", "የመስቀል በዓል"),
            ("2022-10-08", "የመውሊድ በዓል"),
            ("2022-12-09", "የብሔር ብሔረሰቦች ቀን"),
        )

    def test_l10n_ar(self):
        self.assertLocalizedHolidays(
            "ar",
            ("2022-01-07", "عيد الميلاد الإثيوبي (جنا)"),
            ("2022-01-19", "عيد الغطاس الإثيوبي (طمقت)"),
            ("2022-02-20", "يوم الشهداء"),
            ("2022-03-02", "عيد النصر في معركة عدوا"),
            ("2022-04-22", "الجمعة العظيمة (سقلَت)"),
            ("2022-04-24", "عيد القيامة (فاسيكا)"),
            ("2022-05-01", "اليوم العالمي للعمال"),
            ("2022-05-02", "عيد الفطر"),
            ("2022-05-05", "يوم انتصار الوطنيين الإثيوبيين"),
            ("2022-05-28", "يوم سقوط نظام الدرج"),
            ("2022-07-09", "عيد الأضحى"),
            ("2022-09-11", "رأس السنة الإثيوبية (إنكوتاتاش)"),
            ("2022-09-27", "عيد الصليب (مسقل)"),
            ("2022-10-08", "عيد المولد النبوي"),
            ("2022-12-09", "يوم الأمم والقوميات والشعوب"),
        )

    def test_l10n_en_et(self):
        self.assertLocalizedHolidays(
            "en_ET",
            ("2022-01-07", "Christmas Holiday"),
            ("2022-01-19", "Epiphany"),
            ("2022-02-20", "Ethiopian Martyrs' Day"),
            ("2022-03-02", "Adwa Victory Day"),
            ("2022-04-22", "Good Friday"),
            ("2022-04-24", "Easter"),
            ("2022-05-01", "International Workers' Day"),
            ("2022-05-02", "Eid al-Fitr"),
            ("2022-05-05", "Ethiopian Patriots' Victory Day"),
            ("2022-05-28", "Downfall of the Dergue Regime Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-09-11", "New Year (Enkutatash)"),
            ("2022-09-27", "Meskel Holiday"),
            ("2022-10-08", "Mawlid"),
            (
                "2022-12-09",
                "Ethiopian National Unity Day (Ethiopian Nations and Nationalities) Day",
            ),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-07", "Christmas Day"),
            ("2022-01-19", "Epiphany"),
            ("2022-02-20", "Ethiopian Martyrs' Day"),
            ("2022-03-02", "Adwa Victory Day"),
            ("2022-04-22", "Good Friday"),
            ("2022-04-24", "Easter Sunday"),
            ("2022-05-01", "International Workers' Day"),
            ("2022-05-02", "Eid al-Fitr"),
            ("2022-05-05", "Ethiopian Patriots' Victory Day"),
            ("2022-05-28", "Downfall of the Dergue Regime Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-09-11", "Ethiopian New Year"),
            ("2022-09-27", "Finding of True Cross"),
            ("2022-10-08", "Prophet's Birthday"),
            ("2022-12-09", "Nations, Nationalities and Peoples Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.bhutan import Bhutan
from tests.common import CommonCountryTests


class TestBhutan(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Bhutan)

    def test_birthday_anniversary_of_his_majesty_the_king(self):
        name = "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"
        self.assertHolidayName(
            name,
            (f"{year}-02-21" for year in self.full_range),
            (f"{year}-02-22" for year in self.full_range),
            (f"{year}-02-23" for year in self.full_range),
        )

    def test_birth_anniversary_third_druk_gyalpo(self):
        self.assertHolidayName(
            "མི་དབང་འབྲུག་རྒྱལ་གསུམ་པའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།",
            (f"{year}-05-02" for year in self.full_range),
        )

    def test_coronation_his_majesty(self):
        self.assertHolidayName(
            "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་གསེར་ཁྲི་མངའ་གསོལ་གྱི་དུས་ཆེན་ངལ་གསོལ།",
            (f"{year}-11-01" for year in self.full_range),
        )

    def test_birth_anniversary_fourth_druk_gyalpo(self):
        self.assertHolidayName(
            "མི་དབང་འབྲུག་རྒྱལ་བཞི་པའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།",
            (f"{year}-11-11" for year in self.full_range),
        )

    def test_national_day(self):
        self.assertHolidayName(
            "རྒྱལ་ཡོངས་དུས་ཆེན་གྱི་ངལ་གསོལ།", (f"{year}-12-17" for year in self.full_range)
        )

    def test_winter_solstice(self):
        name = "དགུན་ཉི་ལྡོག་གི་ངལ་གསོལ།"
        self.assertHolidayName(
            name,
            (
                f"{year}-01-02"
                for year in (
                    *range(self.start_year, 2019),
                    *range(2020, 2022),
                    *range(2023, self.end_year),
                )
            ),
            "2019-01-03",
            "2022-01-01",
        )

    def test_day_of_offering(self):
        name = "སྔར་སྲོལ་འབུལ་བའི་ལོ་གསར་གྱི་ངལ་གསོལ།"
        self.assertHolidayName(
            name,
            "2020-01-25",
            "2021-01-14",
            "2022-01-03",
            "2023-01-22",
            "2024-01-12",
            "2025-01-30",
        )
        self.assertHolidayName(name, set(self.full_range) - {2031, 2040})

    def test_losar(self):
        name = "གནམ་ལོ་གསར་ཚེས་ཀྱིས་དུས་སྟོན་ངལ་གསོལ།"
        self.assertHolidayName(
            name,
            "2020-02-24",
            "2021-02-12",
            "2022-03-03",
            "2023-02-21",
            "2024-02-11",
            "2025-02-28",
        )
        self.assertHolidayName(name, self.full_range)

    def test_death_of_zhabdrung(self):
        name = "མཐུ་ཆེན་ཆོས་ཀྱི་རྒྱལ་པོའི་དུས་ཆེན་ངལ་གསོལ།"
        self.assertHolidayName(
            name,
            "2021-04-22",
            "2022-05-11",
            "2023-04-30",
            "2024-04-18",
            "2025-05-07",
        )
        self.assertHolidayName(name, set(self.full_range) - {2010, 2011, 2020})

    def test_lord_buddha_parinirvana(self):
        name = "སངས་རྒྱས་བཅོམ་ལྡན་འདས་ཀྱི་དུས་ཆེན་ལྔ་འཛོམས་ངལ་གསོལ།"
        self.assertHolidayName(
            name,
            "2020-06-05",
            "2021-05-26",
            "2022-06-14",
            "2023-06-04",
            "2024-05-23",
            "2025-06-11",
        )
        self.assertHolidayName(name, self.full_range)

    def test_birth_anniversary_guru_rinpoche(self):
        name = "ཨོ་རྒྱན་གུ་རུ་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོ།"
        self.assertHolidayName(
            name,
            "2020-06-30",
            "2021-06-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-07-05",
        )
        self.assertHolidayName(name, range(self.start_year, 2027), range(2028, self.end_year))

    def test_first_sermon_lord_buddha(self):
        name = "སྟོན་པ་མཆོག་ཝ་ར་ན་སིར་བདེན་བཞིའི་ཆོས་འཁོར་བསྐོར་བའི་དུས་ཆེན་ངལ་གསོ།"
        self.assertHolidayName(
            name,
            "2020-07-24",
            "2021-07-14",
            "2022-08-01",
            "2023-07-21",
            "2024-07-10",
            "2025-07-28",
        )
        self.assertHolidayName(name, self.full_range)

    def test_blessed_rainy_day(self):
        name = "རྒྱལ་ཡོངས་ཁྲུས་བབས་ངལ་གསོ།"
        self.assertHolidayName(
            name,
            "2020-09-23",
            "2021-09-23",
            "2022-09-23",
            "2023-09-24",
            "2024-09-23",
            "2025-09-23",
        )
        self.assertHolidayName(name, self.full_range)

    def test_dassain(self):
        name = "ལྷོ་མཚམས་ད་སའིན་ངལ་གསོ།"
        self.assertHolidayName(
            name,
            "2020-10-26",
            "2021-10-15",
            "2022-10-05",
            "2023-10-24",
            "2024-10-12",
            "2025-10-02",
        )
        self.assertHolidayName(name, (*range(self.start_year, 2012), *range(2014, 2025)))

    def test_descending_day_lord_buddha(self):
        name = "རྒྱལ་བ་ལྷ་ལས་བབས་པའི་དུས་ཆེན་ངལ་གསོ།"
        self.assertHolidayName(
            name,
            "2020-11-07",
            "2021-10-27",
            "2022-11-15",
            "2023-11-04",
            "2024-11-22",
            "2025-11-11",
        )
        self.assertHolidayName(name, self.full_range)

    def test_thimphu_drubchoe(self):
        name = "ཐིམ་རྫོང་ལྷ་མོའི་དངོས་འཆམ་མཇལ་རྒྱུའི་ངལ་གསོ།"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "15":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-09-23",
                    "2021-09-12",
                    "2022-10-01",
                    "2023-09-21",
                    "2024-09-10",
                    "2025-09-29",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_thimphu_tshechu(self):
        name = "ཐིམ་ཕུ་ཚེས་བཅུའི་ངལ་གསོ།"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "15":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-09-26",
                    "2020-09-27",
                    "2020-09-28",
                    "2021-09-15",
                    "2021-09-16",
                    "2021-09-17",
                    "2022-10-04",
                    "2022-10-05",
                    "2022-10-06",
                    "2023-09-24",
                    "2023-09-25",
                    "2023-09-26",
                    "2024-09-13",
                    "2024-09-14",
                    "2024-09-15",
                    "2025-10-02",
                    "2025-10-03",
                    "2025-10-04",
                )
                self.assertHolidayNameCount(name, 3, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-02", "དགུན་ཉི་ལྡོག་གི་ངལ་གསོལ།"),
            ("2025-01-30", "སྔར་སྲོལ་འབུལ་བའི་ལོ་གསར་གྱི་ངལ་གསོལ།"),
            ("2025-02-21", "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"),
            ("2025-02-22", "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"),
            ("2025-02-23", "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"),
            ("2025-02-28", "གནམ་ལོ་གསར་ཚེས་ཀྱིས་དུས་སྟོན་ངལ་གསོལ།"),
            ("2025-03-01", "གནམ་ལོ་གསར་ཚེས་ཀྱིས་དུས་སྟོན་ངལ་གསོལ།"),
            ("2025-05-02", "མི་དབང་འབྲུག་རྒྱལ་གསུམ་པའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"),
            ("2025-05-07", "མཐུ་ཆེན་ཆོས་ཀྱི་རྒྱལ་པོའི་དུས་ཆེན་ངལ་གསོལ།"),
            ("2025-06-11", "སངས་རྒྱས་བཅོམ་ལྡན་འདས་ཀྱི་དུས་ཆེན་ལྔ་འཛོམས་ངལ་གསོལ།"),
            ("2025-07-05", "ཨོ་རྒྱན་གུ་རུ་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོ།"),
            ("2025-07-28", "སྟོན་པ་མཆོག་ཝ་ར་ན་སིར་བདེན་བཞིའི་ཆོས་འཁོར་བསྐོར་བའི་དུས་ཆེན་ངལ་གསོ།"),
            ("2025-09-23", "རྒྱལ་ཡོངས་ཁྲུས་བབས་ངལ་གསོ།"),
            ("2025-10-02", "ལྷོ་མཚམས་ད་སའིན་ངལ་གསོ།"),
            ("2025-11-01", "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་གསེར་ཁྲི་མངའ་གསོལ་གྱི་དུས་ཆེན་ངལ་གསོལ།"),
            (
                "2025-11-11",
                "མི་དབང་འབྲུག་རྒྱལ་བཞི་པའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།; རྒྱལ་བ་ལྷ་ལས་བབས་པའི་དུས་ཆེན་ངལ་གསོ།",
            ),
            ("2025-12-17", "རྒྱལ་ཡོངས་དུས་ཆེན་གྱི་ངལ་གསོལ།"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-02", "དགུན་ཉི་ལྡོག་གི་ངལ་གསོལ།"),
            ("2024-01-12", "སྔར་སྲོལ་འབུལ་བའི་ལོ་གསར་གྱི་ངལ་གསོལ།"),
            ("2024-02-10", "གནམ་ལོ་གསར་ཚེས་ཀྱིས་དུས་སྟོན་ངལ་གསོལ།"),
            ("2024-02-11", "གནམ་ལོ་གསར་ཚེས་ཀྱིས་དུས་སྟོན་ངལ་གསོལ།"),
            ("2024-02-21", "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"),
            ("2024-02-22", "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"),
            ("2024-02-23", "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"),
            ("2024-04-18", "མཐུ་ཆེན་ཆོས་ཀྱི་རྒྱལ་པོའི་དུས་ཆེན་ངལ་གསོལ།"),
            ("2024-05-02", "མི་དབང་འབྲུག་རྒྱལ་གསུམ་པའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"),
            ("2024-05-23", "སངས་རྒྱས་བཅོམ་ལྡན་འདས་ཀྱི་དུས་ཆེན་ལྔ་འཛོམས་ངལ་གསོལ།"),
            ("2024-06-16", "ཨོ་རྒྱན་གུ་རུ་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོ།"),
            ("2024-07-10", "སྟོན་པ་མཆོག་ཝ་ར་ན་སིར་བདེན་བཞིའི་ཆོས་འཁོར་བསྐོར་བའི་དུས་ཆེན་ངལ་གསོ།"),
            ("2024-09-10", "ཐིམ་རྫོང་ལྷ་མོའི་དངོས་འཆམ་མཇལ་རྒྱུའི་ངལ་གསོ།"),
            ("2024-09-13", "ཐིམ་ཕུ་ཚེས་བཅུའི་ངལ་གསོ།"),
            ("2024-09-14", "ཐིམ་ཕུ་ཚེས་བཅུའི་ངལ་གསོ།"),
            ("2024-09-15", "ཐིམ་ཕུ་ཚེས་བཅུའི་ངལ་གསོ།"),
            ("2024-09-23", "རྒྱལ་ཡོངས་ཁྲུས་བབས་ངལ་གསོ།"),
            ("2024-10-12", "ལྷོ་མཚམས་ད་སའིན་ངལ་གསོ།"),
            ("2024-11-01", "མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་གསེར་ཁྲི་མངའ་གསོལ་གྱི་དུས་ཆེན་ངལ་གསོལ།"),
            ("2024-11-11", "མི་དབང་འབྲུག་རྒྱལ་བཞི་པའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"),
            ("2024-11-22", "རྒྱལ་བ་ལྷ་ལས་བབས་པའི་དུས་ཆེན་ངལ་གསོ།"),
            ("2024-12-17", "རྒྱལ་ཡོངས་དུས་ཆེན་གྱི་ངལ་གསོལ།"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-02", "Winter Solstice"),
            ("2024-01-12", "Traditional Day of Offering"),
            ("2024-02-10", "Losar"),
            ("2024-02-11", "Losar"),
            ("2024-02-21", "Birth Anniversary of His Majesty the King"),
            ("2024-02-22", "Birth Anniversary of His Majesty the King"),
            ("2024-02-23", "Birth Anniversary of His Majesty the King"),
            ("2024-04-18", "Death Anniversary of Zhabdrung"),
            ("2024-05-02", "Birth Anniversary of the 3rd Druk Gyalpo"),
            ("2024-05-23", "Lord Buddha's Parinirvana"),
            ("2024-06-16", "Birth Anniversary of Guru Rinpoche"),
            ("2024-07-10", "First Sermon of Lord Buddha"),
            ("2024-09-10", "Thimphu Drubchoe"),
            ("2024-09-13", "Thimphu Tshechu"),
            ("2024-09-14", "Thimphu Tshechu"),
            ("2024-09-15", "Thimphu Tshechu"),
            ("2024-09-23", "Blessed Rainy Day"),
            ("2024-10-12", "Dassain"),
            ("2024-11-01", "Coronation of His Majesty the King"),
            ("2024-11-11", "Birth Anniversary of the 4th Druk Gyalpo - Constitution Day"),
            ("2024-11-22", "Descending Day of Lord Buddha"),
            ("2024-12-17", "National Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.kiribati import Kiribati
from tests.common import CommonCountryTests


class TestKiribati(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Kiribati)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_womens_day(self):
        name = "International Women's Day"
        self.assertHolidayName(name, (f"{year}-03-08" for year in range(2003, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2003))
        obs_dts = (
            "2014-03-10",
            "2015-03-09",
            "2020-03-09",
            "2025-03-10",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Holy Saturday"
        self.assertHolidayName(
            name,
            "1999-04-03",
            "2000-04-22",
            "2001-04-14",
            "2002-03-30",
        )
        self.assertHolidayName(name, range(self.start_year, 2003))
        self.assertNoHolidayName(name, range(2003, self.end_year))

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_health_day(self):
        name = "National Health Day"
        self.assertHolidayName(name, (f"{year}-04-19" for year in range(2003, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2003))
        obs_dts = (
            "2014-04-21",
            "2015-04-20",
            "2020-04-20",
            "2025-04-21",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)

    def test_public_holiday_may_9(self):
        name = "Public Holiday"
        self.assertHolidayName(name, (f"{year}-05-09" for year in range(1993, 2003)))
        self.assertNoHolidayName(
            name,
            (
                f"{year}-05-09"
                for year in (*range(self.start_year, 1993), *range(2003, self.end_year))
            ),
        )
        obs_dts = (
            "1993-05-10",
            "1998-05-11",
            "1999-05-10",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_gospel_day(self):
        name = "Gospel Day"
        self.assertHolidayName(name, (f"{year}-07-11" for year in range(2002, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2002))

    def test_national_day_anniversary(self):
        name = "National Day - Independence Anniversary"
        self.assertHolidayName(name, (f"{year}-07-12" for year in range(1993, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1993))
        obs_dts = (
            "2014-07-14",
            "2015-07-13",
            "2020-07-13",
            "2025-07-14",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_day_unimwane(self):
        name = "National Day (in honor of Unimwane)"
        self.assertHolidayName(name, (f"{year}-07-15" for year in range(2002, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2002))

    def test_national_day_unaine(self):
        name = "National Day (in honor of Unaine)"
        self.assertHolidayName(name, (f"{year}-07-16" for year in range(2002, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2002))
        obs_dts = (
            "2011-07-18",
            "2016-07-18",
            "2017-07-17",
            "2022-07-18",
            "2023-07-17",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_public_holiday_1st_mon_of_aug(self):
        name = "Public Holiday"
        self.assertHolidayName(
            name,
            "1999-08-02",
            "2000-08-07",
            "2001-08-06",
            "2002-08-05",
        )
        self.assertHolidayName(name, range(self.start_year, 2003))
        self.assertNoHolidayName(name, range(2003, self.end_year))

    def test_youth_day(self):
        name = "Youth Day"
        self.assertHolidayName(name, (f"{year}-08-05" for year in range(2003, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2003))
        obs_dts = (
            "2012-08-06",
            "2017-08-07",
            "2018-08-06",
            "2023-08-07",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_public_holiday_nov_10(self):
        name = "Public Holiday"
        self.assertHolidayName(name, (f"{year}-11-10" for year in range(self.start_year, 1993)))
        self.assertNoHolidayName(name, (f"{year}-11-10" for year in range(1993, self.end_year)))
        obs_dts = (
            "1985-11-11",
            "1990-11-12",
            "1991-11-11",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_human_rights_peace_day(self):
        name = "Human Rights and Peace Day"
        self.assertHolidayName(
            name,
            (f"{year}-12-10" for year in range(1993, 2003)),
            (f"{year}-12-09" for year in range(2003, self.end_year)),
        )
        self.assertNoHolidayName(name, range(self.start_year, 1993))
        obs_dts = (
            "1995-12-11",
            "2000-12-11",
            "2006-12-11",
            "2007-12-10",
            "2012-12-10",
            "2017-12-11",
            "2018-12-10",
            "2023-12-11",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-03-08", "International Women's Day"),
            ("2025-03-10", "International Women's Day (observed)"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-19", "National Health Day"),
            ("2025-04-21", "Easter Monday; National Health Day (observed)"),
            ("2025-07-11", "Gospel Day"),
            ("2025-07-12", "National Day - Independence Anniversary"),
            ("2025-07-14", "National Day - Independence Anniversary (observed)"),
            ("2025-07-15", "National Day (in honor of Unimwane)"),
            ("2025-07-16", "National Day (in honor of Unaine)"),
            ("2025-08-05", "Youth Day"),
            ("2025-12-09", "Human Rights and Peace Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.micronesia import Micronesia
from tests.common import CommonCountryTests


class TestMicronesia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1987, 2050)
        super().setUpClass(Micronesia, years=years, years_non_observed=years)
        cls.subdiv_holidays = {
            subdiv: Micronesia(subdiv=subdiv, years=years) for subdiv in Micronesia.subdivisions
        }
        cls.subdiv_holidays_non_observed = {
            subdiv: Micronesia(subdiv=subdiv, years=years, observed=False)
            for subdiv in Micronesia.subdivisions
        }

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1987, 2050)))
        dt = (
            "2010-12-31",
            "2012-01-02",
            "2017-01-02",
            "2021-12-31",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_micronesian_culture_and_tradition_day(self):
        name = "Micronesian Culture and Tradition Day"
        self.assertHolidayName(name, (f"{year}-03-31" for year in range(2011, 2050)))
        self.assertNoHolidayName(name, range(1987, 2011))
        dt = (
            "2012-03-30",
            "2013-04-01",
            "2018-03-30",
            "2019-04-01",
            "2024-04-01",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_federated_states_of_micronesia_day(self):
        name = "Federated States of Micronesia Day"
        self.assertHolidayName(name, (f"{year}-05-10" for year in range(1987, 2050)))
        dt = (
            "2003-05-09",
            "2008-05-09",
            "2009-05-11",
            "2014-05-09",
            "2015-05-11",
            "2020-05-11",
            "2025-05-09",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_united_nations_day(self):
        name = "United Nations Day"
        self.assertHolidayName(name, (f"{year}-10-24" for year in range(1991, 2050)))
        self.assertNoHolidayName(name, range(1987, 1991))
        dt = (
            "2004-10-25",
            "2009-10-23",
            "2010-10-25",
            "2015-10-23",
            "2020-10-23",
            "2021-10-25",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-11-03" for year in range(1987, 2050)))
        dt = (
            "2001-11-02",
            "2002-11-04",
            "2007-11-02",
            "2012-11-02",
            "2013-11-04",
            "2018-11-02",
            "2019-11-04",
            "2024-11-04",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_fsm_veterans_of_foreign_wars_day(self):
        name = "FSM Veterans of Foreign Wars Day"
        self.assertHolidayName(name, (f"{year}-11-11" for year in range(2004, 2050)))
        self.assertNoHolidayName(name, range(1987, 2004))
        dt = (
            "2006-11-10",
            "2007-11-12",
            "2012-11-12",
            "2017-11-10",
            "2018-11-12",
            "2023-11-10",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_presidents_day(self):
        name = "Presidents Day"
        self.assertHolidayName(name, (f"{year}-11-23" for year in range(2021, 2050)))
        self.assertNoHolidayName(name, range(1987, 2021))
        dt = (
            "2024-11-22",
            "2025-11-24",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(1987, 2050)))
        dt = (
            "2004-12-24",
            "2005-12-26",
            "2010-12-24",
            "2011-12-26",
            "2016-12-26",
            "2021-12-24",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_state_charter_day(self):
        name = "State Charter Day"
        self.assertNoHolidayName(name)
        dt = (
            "2004-09-27",
            "2009-09-25",
            "2010-09-27",
            "2015-09-25",
            "2020-09-25",
            "2021-09-27",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "TRK":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-26" for year in range(1987, 2050))
                )
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_chuuk_state_constitution_day(self):
        name = "Chuuk State Constitution Day"
        self.assertNoHolidayName(name)
        dt = (
            "2000-10-02",
            "2005-09-30",
            "2006-10-02",
            "2011-09-30",
            "2016-09-30",
            "2017-10-02",
            "2022-09-30",
            "2023-10-02",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "TRK":
                self.assertHolidayName(
                    name, holidays, (f"{year}-10-01" for year in range(1990, 2050))
                )
                self.assertNoHolidayName(name, holidays, range(1987, 1990))
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_kosrae_state_constitution_day(self):
        name = "Kosrae State Constitution Day"
        self.assertNoHolidayName(name)
        dt = (
            "2003-01-10",
            "2004-01-12",
            "2009-01-12",
            "2014-01-10",
            "2015-01-12",
            "2020-01-10",
            "2025-01-10",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "KSA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-11" for year in range(1991, 2050))
                )
                self.assertNoHolidayName(name, holidays, range(1987, 1991))
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertNoHolidayName(name)
        dt = (
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                case "KSA":
                    self.assertHolidayName(name, holidays, dt)
                    self.assertHolidayName(name, holidays, range(2000, 2050))
                    self.assertNoHolidayName(name, holidays, range(1987, 2000))
                case "PNI":
                    self.assertHolidayName(name, holidays, dt)
                    self.assertHolidayName(name, holidays, range(1987, 2050))
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_gospel_day(self):
        name = "Gospel Day"
        self.assertNoHolidayName(name)
        dt = (
            "2004-08-20",
            "2005-08-22",
            "2010-08-20",
            "2011-08-22",
            "2016-08-22",
            "2021-08-20",
            "2022-08-22",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "KSA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-21" for year in range(2000, 2050))
                )
                self.assertNoHolidayName(name, holidays, range(1987, 2000))
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_kosrae_liberation_day(self):
        name = "Kosrae Liberation Day"
        self.assertNoHolidayName(name)
        dt = (
            "2001-09-07",
            "2002-09-09",
            "2007-09-07",
            "2012-09-07",
            "2013-09-09",
            "2018-09-07",
            "2019-09-09",
            "2024-09-09",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "KSA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-08" for year in range(1991, 2050))
                )
                self.assertNoHolidayName(name, holidays, range(1987, 1991))
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_self_government_day(self):
        name = "Self Government Day"
        self.assertNoHolidayName(name)
        dt = (
            "2001-11-02",
            "2002-11-04",
            "2007-11-02",
            "2012-11-02",
            "2013-11-04",
            "2018-11-02",
            "2019-11-04",
            "2024-11-04",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "KSA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-03" for year in range(1987, 2050))
                )
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_thanksgiving_day(self):
        name = "Thanksgiving Day"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "KSA":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-11-26",
                    "2021-11-25",
                    "2022-11-24",
                    "2023-11-23",
                    "2024-11-28",
                    "2025-11-27",
                )
                self.assertHolidayName(name, holidays, range(2011, 2050))
                self.assertNoHolidayName(name, holidays, range(1987, 2011))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_kosrae_disability_day(self):
        name = "Kosrae Disability Day"
        self.assertNoHolidayName(name)
        dt = (
            "2028-12-04",
            "2033-12-02",
            "2034-12-04",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "KSA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-03" for year in range(2024, 2050))
                )
                self.assertNoHolidayName(name, holidays, range(1987, 2024))
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_womens_day(self):
        name = "Women's Day"
        self.assertNoHolidayName(name)
        dt = ("2025-03-07",)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PNI":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-08" for year in range(2022, 2050))
                )
                self.assertNoHolidayName(name, holidays, range(1987, 2022))
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_pohnpei_cultural_day(self):
        name = "Pohnpei Cultural Day"
        self.assertNoHolidayName(name)
        dt = (
            "2001-03-30",
            "2002-04-01",
            "2007-03-30",
            "2012-03-30",
            "2013-04-01",
            "2019-04-01",
            "2024-04-01",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PNI":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-31" for year in range(1987, 2050))
                )
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_liberation_day(self):
        name = "Liberation Day"
        self.assertNoHolidayName(name)
        dt = (
            "2004-09-10",
            "2005-09-12",
            "2010-09-10",
            "2011-09-12",
            "2016-09-12",
            "2021-09-10",
            "2022-09-12",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PNI":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-11" for year in range(1987, 2050))
                )
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_pohnpei_constitution_day(self):
        name = "Pohnpei Constitution Day"
        self.assertNoHolidayName(name)
        dt = (
            "2003-11-07",
            "2008-11-07",
            "2009-11-09",
            "2014-11-07",
            "2015-11-09",
            "2020-11-09",
            "2025-11-07",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PNI":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-08" for year in range(1987, 2050))
                )
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_yap_day(self):
        name = "Yap Day"
        self.assertNoHolidayName(name)
        dt = (
            "2003-02-28",
            "2008-02-29",
            "2009-03-02",
            "2014-02-28",
            "2015-03-02",
            "2020-03-02",
            "2025-02-28",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "YAP":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-01" for year in range(1987, 2050))
                )
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_yap_state_constitution_day(self):
        name = "Yap State Constitution Day"
        self.assertNoHolidayName(name)
        dt = (
            "2000-12-25",
            "2005-12-23",
            "2006-12-25",
            "2011-12-23",
            "2016-12-23",
            "2017-12-25",
            "2022-12-23",
            "2023-12-25",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "YAP":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-24" for year in range(1987, 2050))
                )
                self.assertHolidayName(f"{name} (observed)", holidays, dt)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_2024(self):
        # https://www.timeanddate.com/holidays/micronesia/2024?hol=1
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-03-31", "Micronesian Culture and Tradition Day"),
            ("2024-04-01", "Micronesian Culture and Tradition Day (observed)"),
            ("2024-05-10", "Federated States of Micronesia Day"),
            ("2024-10-24", "United Nations Day"),
            ("2024-11-03", "Independence Day"),
            ("2024-11-04", "Independence Day (observed)"),
            ("2024-11-11", "FSM Veterans of Foreign Wars Day"),
            ("2024-11-22", "Presidents Day (observed)"),
            ("2024-11-23", "Presidents Day"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-01-10", "Kosrae State Constitution Day (observed)"),
            ("2025-01-11", "Kosrae State Constitution Day"),
            ("2025-02-28", "Yap Day (observed)"),
            ("2025-03-01", "Yap Day"),
            ("2025-03-07", "Women's Day (observed)"),
            ("2025-03-08", "Women's Day"),
            ("2025-03-31", "Micronesian Culture and Tradition Day; Pohnpei Cultural Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-05-09", "Federated States of Micronesia Day (observed)"),
            ("2025-05-10", "Federated States of Micronesia Day"),
            ("2025-08-21", "Gospel Day"),
            ("2025-09-08", "Kosrae Liberation Day"),
            ("2025-09-11", "Liberation Day"),
            ("2025-09-26", "State Charter Day"),
            ("2025-10-01", "Chuuk State Constitution Day"),
            ("2025-10-24", "United Nations Day"),
            ("2025-11-03", "Independence Day; Self Government Day"),
            ("2025-11-07", "Pohnpei Constitution Day (observed)"),
            ("2025-11-08", "Pohnpei Constitution Day"),
            ("2025-11-11", "FSM Veterans of Foreign Wars Day"),
            ("2025-11-23", "Presidents Day"),
            ("2025-11-24", "Presidents Day (observed)"),
            ("2025-11-27", "Thanksgiving Day"),
            ("2025-12-03", "Kosrae Disability Day"),
            ("2025-12-24", "Yap State Constitution Day"),
            ("2025-12-25", "Christmas Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-10", "Kosrae State Constitution Day (observed)"),
            ("2025-01-11", "Kosrae State Constitution Day"),
            ("2025-02-28", "Yap Day (observed)"),
            ("2025-03-01", "Yap Day"),
            ("2025-03-07", "Women's Day (observed)"),
            ("2025-03-08", "Women's Day"),
            ("2025-03-31", "Micronesian Culture and Tradition Day; Pohnpei Cultural Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-05-09", "Federated States of Micronesia Day (observed)"),
            ("2025-05-10", "Federated States of Micronesia Day"),
            ("2025-08-21", "Gospel Day"),
            ("2025-09-08", "Kosrae Liberation Day"),
            ("2025-09-11", "Liberation Day"),
            ("2025-09-26", "State Charter Day"),
            ("2025-10-01", "Chuuk State Constitution Day"),
            ("2025-10-24", "United Nations Day"),
            ("2025-11-03", "Independence Day; Self Government Day"),
            ("2025-11-07", "Pohnpei Constitution Day (observed)"),
            ("2025-11-08", "Pohnpei Constitution Day"),
            ("2025-11-11", "FSM Veterans of Foreign Wars Day"),
            ("2025-11-23", "Presidents Day"),
            ("2025-11-24", "Presidents Day (observed)"),
            ("2025-11-27", "Thanksgiving Day"),
            ("2025-12-03", "Kosrae Disability Day"),
            ("2025-12-24", "Yap State Constitution Day"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import HINDU
from holidays.countries.kenya import Kenya
from tests.common import CommonCountryTests


class TestKenya(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Kenya)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Kenya(categories=HINDU, years=range(self.start_year, 1984)))

    def test_special_holidays(self):
        self.assertHoliday(
            "2015-11-26",
            "2017-08-08",
            "2017-10-25",
            "2017-10-26",
            "2017-11-28",
            "2020-02-11",
            "2022-04-29",
            "2022-08-09",
            "2022-09-10",
            "2022-09-11",
            "2022-09-12",
            "2022-09-13",
            "2023-11-13",
            "2024-05-10",
            "2024-11-01",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_madaraka_day(self):
        name = "Madaraka Day"
        self.assertHolidayName(name, (f"{year}-06-01" for year in range(2011, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2011))
        obs_dts = (
            "2014-06-02",
            "2025-06-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_mazingira_day(self):
        name_1990 = "Moi Day"
        name_2021 = "Utamaduni Day"
        name_2025 = "Mazingira Day"

        self.assertHolidayName(
            name_1990, (f"{year}-10-10" for year in (*range(1990, 2010), *range(2018, 2021)))
        )
        self.assertHolidayName(name_2021, (f"{year}-10-10" for year in range(2021, 2025)))
        self.assertHolidayName(name_2025, (f"{year}-10-10" for year in range(2025, self.end_year)))
        self.assertNoHolidayName(
            name_1990, range(self.start_year, 1990), range(2010, 2018), range(2021, self.end_year)
        )
        self.assertNoHolidayName(
            name_2021, range(self.start_year, 2021), range(2025, self.end_year)
        )
        self.assertNoHolidayName(name_2025, range(self.start_year, 2025))
        dts_1990 = ("2004-10-11",)
        self.assertHolidayName(f"{name_1990} (observed)", dts_1990)
        dts_2021 = ("2021-10-11",)
        self.assertHolidayName(f"{name_2021} (observed)", dts_2021)
        dts_2025 = ("2027-10-11",)
        self.assertHolidayName(f"{name_2025} (observed)", dts_2025)
        self.assertNoNonObservedHoliday(dts_1990, dts_2021, dts_2025)

    def test_mashujaa_day(self):
        name_1964 = "Kenyatta Day"
        name_2011 = "Mashujaa Day"
        self.assertHolidayName(
            name_1964, (f"{year}-10-20" for year in range(self.start_year, 2011))
        )
        self.assertHolidayName(name_2011, (f"{year}-10-20" for year in range(2011, self.end_year)))
        self.assertNoHolidayName(name_1964, range(2011, self.end_year))
        self.assertNoHolidayName(name_2011, range(self.start_year, 2011))
        dts_1964 = ("2002-10-21",)
        self.assertHolidayName(f"{name_1964} (observed)", dts_1964)
        dts_2011 = (
            "2013-10-21",
            "2019-10-21",
            "2024-10-21",
        )
        self.assertHolidayName(f"{name_2011} (observed)", dts_2011)
        self.assertNoNonObservedHoliday(dts_1964, dts_2011)

    def test_jamhury_day(self):
        name_1964 = "Independence Day"
        name_2011 = "Jamhuri Day"
        self.assertHolidayName(
            name_1964, (f"{year}-12-12" for year in range(self.start_year, 2011))
        )
        self.assertHolidayName(name_2011, (f"{year}-12-12" for year in range(2011, self.end_year)))
        self.assertNoHolidayName(name_1964, range(2011, self.end_year))
        self.assertNoHolidayName(name_2011, range(self.start_year, 2011))
        dts_1964 = (
            "2004-12-13",
            "2010-12-13",
        )
        self.assertHolidayName(f"{name_1964} (observed)", dts_1964)
        dts_2011 = (
            "2021-12-13",
            "2027-12-13",
        )
        self.assertHolidayName(f"{name_2011} (observed)", dts_2011)
        self.assertNoNonObservedHoliday(dts_1964, dts_2011)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2005-12-27",
            "2011-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Idd-ul-Fitr"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2019-06-05",
            "2020-05-25",
            "2021-05-14",
            "2022-05-03",
            "2023-04-21",
            "2024-04-10",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2001-12-17",
            "2004-11-15",
            "2009-09-21",
            "2012-08-20",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoIslamicNoEstimatedNonObservedHoliday(obs_dts)

    def test_diwali(self):
        name = "Diwali"
        self.assertNoHolidayName(name)
        self.assertHinduHolidayName(
            name,
            "2019-10-28",
            "2020-11-14",
            "2021-11-04",
            "2022-10-24",
            "2023-11-12",
            "2024-10-31",
        )
        self.assertHinduHolidayName(name, range(1984, self.end_year))
        self.assertNoHinduHolidayName(name, range(self.start_year, 1984))

    def test_eid_al_adha(self):
        name = "Idd-ul-Azha"
        self.assertNoHolidayName(name)
        self.assertIslamicIslamicNoEstimatedHolidayName(
            name,
            "2019-08-12",
            "2020-07-31",
            "2021-07-20",
            "2022-07-11",
            "2023-06-28",
            "2024-06-17",
        )
        self.assertIslamicIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_2010(self):
        self.assertHolidaysInYear(
            2010,
            ("2010-01-01", "New Year's Day"),
            ("2010-04-02", "Good Friday"),
            ("2010-04-05", "Easter Monday"),
            ("2010-05-01", "Labour Day"),
            ("2010-09-10", "Idd-ul-Fitr (estimated)"),
            ("2010-10-20", "Kenyatta Day"),
            ("2010-12-12", "Independence Day"),
            ("2010-12-13", "Independence Day (observed)"),
            ("2010-12-25", "Christmas Day"),
            ("2010-12-26", "Boxing Day"),
            ("2010-12-27", "Boxing Day (observed)"),
        )

    def test_2019(self):
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "New Year's Day"),
            ("2019-04-19", "Good Friday"),
            ("2019-04-22", "Easter Monday"),
            ("2019-05-01", "Labour Day"),
            ("2019-06-01", "Madaraka Day"),
            ("2019-06-05", "Idd-ul-Fitr"),
            ("2019-10-10", "Moi Day"),
            ("2019-10-20", "Mashujaa Day"),
            ("2019-10-21", "Mashujaa Day (observed)"),
            ("2019-12-12", "Jamhuri Day"),
            ("2019-12-25", "Christmas Day"),
            ("2019-12-26", "Boxing Day"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-29", "State Funeral for Former President Mwai Kibaki"),
            ("2022-05-01", "Labour Day"),
            ("2022-05-02", "Labour Day (observed)"),
            ("2022-05-03", "Idd-ul-Fitr"),
            ("2022-06-01", "Madaraka Day"),
            ("2022-08-09", "Election Day"),
            ("2022-09-10", "Day of Mourning for Queen Elizabeth II"),
            ("2022-09-11", "Day of Mourning for Queen Elizabeth II"),
            ("2022-09-12", "Day of Mourning for Queen Elizabeth II"),
            ("2022-09-13", "Inauguration Day"),
            ("2022-10-10", "Utamaduni Day"),
            ("2022-10-20", "Mashujaa Day"),
            ("2022-12-12", "Jamhuri Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-10", "Idd-ul-Fitr"),
            ("2024-05-01", "Labour Day"),
            ("2024-05-10", "National Tree Growing Day"),
            ("2024-06-01", "Madaraka Day"),
            ("2024-06-17", "Idd-ul-Azha"),
            ("2024-10-10", "Utamaduni Day"),
            ("2024-10-20", "Mashujaa Day"),
            ("2024-10-21", "Mashujaa Day (observed)"),
            ("2024-10-31", "Diwali"),
            ("2024-11-01", "Inauguration Day"),
            ("2024-12-12", "Jamhuri Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-10", "Eid-al-Fitr"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-10", "National Tree Growing Day"),
            ("2024-06-01", "Madaraka Day"),
            ("2024-06-17", "Eid-al-Adha"),
            ("2024-10-10", "Utamaduni Day"),
            ("2024-10-20", "Mashujaa Day"),
            ("2024-10-21", "Mashujaa Day (observed)"),
            ("2024-10-31", "Diwali"),
            ("2024-11-01", "Inauguration Day"),
            ("2024-12-12", "Jamhuri Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_sw(self):
        self.assertLocalizedHolidays(
            "sw",
            ("2024-01-01", "Siku ya Mwaka Mpya"),
            ("2024-03-29", "Ijumaa Kuu"),
            ("2024-04-01", "Jumatatu ya Pasaka"),
            ("2024-04-10", "Sikukuu ya Idd-ul-Fitr"),
            ("2024-05-01", "Siku ya Kazi"),
            ("2024-05-10", "Siku ya Kitaifa ya Kupanda Miti"),
            ("2024-06-01", "Siku ya Madaraka"),
            ("2024-06-17", "Sikukuu ya Idd-ul-Azha"),
            ("2024-10-10", "Siku ya Utamaduni"),
            ("2024-10-20", "Siku ya Mashujaa"),
            ("2024-10-21", "Siku ya Mashujaa (imezingatiwa)"),
            ("2024-10-31", "Diwali"),
            ("2024-11-01", "Siku ya Uzinduzi"),
            ("2024-12-12", "Siku ya Jamhuri"),
            ("2024-12-25", "Siku ya Krismasi"),
            ("2024-12-26", "Siku ya Ndondi"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date

from holidays.calendars.gregorian import MAR, APR

NYEPI = "NYEPI"


class _BalineseSakaLunar:
    """
    Balinese Saka lunar calendar.

    The Balinese saka calendar is one of two calendars used on the Indonesian island
    of Bali. Unlike the 210-day pawukon calendar, it is based on the phases of the Moon,
    and is approximately the same length as the tropical year (solar year, Gregorian year).
    https://en.wikipedia.org/wiki/Balinese_saka_calendar
    """

    NYEPI_DATES = {
        1983: (MAR, 15),
        1984: (MAR, 4),
        1985: (MAR, 22),
        1986: (MAR, 12),
        1987: (MAR, 31),
        1988: (MAR, 19),
        1989: (MAR, 9),
        1990: (MAR, 27),
        1991: (MAR, 17),
        1992: (MAR, 5),
        1993: (MAR, 24),
        1994: (MAR, 12),
        1995: (APR, 1),
        1996: (MAR, 21),
        1997: (APR, 9),
        1998: (MAR, 29),
        1999: (MAR, 18),
        2000: (APR, 4),
        2001: (MAR, 25),
        2002: (APR, 13),
        2003: (APR, 2),
        2004: (MAR, 22),
        2005: (MAR, 11),
        2006: (MAR, 30),
        2007: (MAR, 19),
        2008: (MAR, 7),
        2009: (MAR, 26),
        2010: (MAR, 16),
        2011: (MAR, 5),
        2012: (MAR, 23),
        2013: (MAR, 12),
        2014: (MAR, 31),
        2015: (MAR, 21),
        2016: (MAR, 9),
        2017: (MAR, 28),
        2018: (MAR, 17),
        2019: (MAR, 7),
        2020: (MAR, 25),
        2021: (MAR, 14),
        2022: (MAR, 3),
        2023: (MAR, 22),
        2024: (MAR, 11),
        2025: (MAR, 29),
        2026: (MAR, 19),
        2027: (MAR, 8),
        2028: (MAR, 26),
        2029: (MAR, 15),
        2030: (MAR, 5),
        2031: (MAR, 24),
        2032: (MAR, 12),
        2033: (MAR, 31),
        2034: (MAR, 20),
        2035: (MAR, 10),
        2036: (MAR, 28),
        2037: (MAR, 17),
        2038: (MAR, 6),
        2039: (MAR, 25),
        2040: (MAR, 14),
        2041: (MAR, 3),
        2042: (MAR, 22),
        2043: (MAR, 11),
        2044: (MAR, 29),
        2045: (MAR, 19),
        2046: (MAR, 8),
        2047: (MAR, 27),
        2048: (MAR, 15),
        2049: (MAR, 5),
        2050: (MAR, 24),
    }

    def _get_holiday(self, holiday: str, year: int) -> date | None:
        dt = getattr(self, f"{holiday}_DATES", {}).get(year, ())
        return date(year, *dt) if dt else None

    def nyepi_date(self, year: int) -> date | None:
        """
        Data References:
            * [1983-2025](https://id.wikipedia.org/wiki/Indonesia_dalam_tahun_1983)
            * [2020-2050](https://web.archive.org/web/20240718011857/https://www.balitrips.com/balinese-temples-ceremony)
        """
        return self._get_holiday(NYEPI, year)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Mozambique(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Mozambique holidays."""

    country = "MZ"
    default_language = "pt_MZ"
    # %s (observed).
    observed_label = tr("%s (ponte)")
    supported_languages = ("en_US", "pt_MZ", "uk")
    start_year = 1975

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # International Fraternalism Day.
        self._add_observed(self._add_new_years_day(tr("Dia da Fraternidade universal")))

        # Heroes' Day.
        self._add_observed(self._add_holiday_feb_3(tr("Dia dos Heróis Moçambicanos")))

        # Women's Day.
        self._add_observed(self._add_holiday_apr_7(tr("Dia da Mulher Moçambicana")))

        # International Workers' Day.
        self._add_observed(self._add_labor_day(tr("Dia Internacional dos Trabalhadores")))

        # Independence Day.
        self._add_observed(self._add_holiday_jun_25(tr("Dia da Independência Nacional")))

        # Victory Day.
        self._add_observed(self._add_holiday_sep_7(tr("Dia da Vitória")))

        self._add_observed(
            # Armed Forces Day.
            self._add_holiday_sep_25(tr("Dia das Forças Armadas de Libertação Nacional"))
        )

        if self._year >= 1993:
            # Peace and Reconciliation Day.
            self._add_observed(self._add_holiday_oct_4(tr("Dia da Paz e Reconciliação")))

        # Family Day.
        self._add_observed(self._add_christmas_day(tr("Dia da Família")))


class MZ(Mozambique):
    pass


class MOZ(Mozambique):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import GREGORIAN_CALENDAR
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.constants import CATHOLIC, ORTHODOX, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Palestine(HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Palestine holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Palestine>
        * <https://web.archive.org/web/20250705114216/https://info.wafa.ps/pages/details/29601>
        * [Declaration of Palestine independence](https://en.wikipedia.org/wiki/Palestine#Uprising,_declaration_and_peace_treaty)
    """

    country = "PS"
    default_language = "ar"
    # %s (estimated).
    estimated_label = tr("%s (تقديري)")
    # State of Palestine declared in November 1988.
    start_year = 1989
    supported_categories = (CATHOLIC, ORTHODOX, PUBLIC)
    supported_languages = ("ar", "en_US")

    def __init__(self, *args, islamic_show_estimated: bool = False, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self, calendar=JULIAN_CALENDAR)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=PalestineIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("رأس السنة الميلادي"))

        # Orthodox Christmas Day.
        self._add_christmas_day(tr("عيد الميلاد المجيد الشرقي"))

        # International Women's Day.
        self._add_womens_day(tr("يوم المراة العالمي"))

        # Easter.
        self._add_easter_sunday(tr("عيد الفصح المجيد"), GREGORIAN_CALENDAR)

        # Easter.
        self._add_easter_sunday(tr("عيد الفصح المجيد"))

        # Labor Day.
        self._add_labor_day(tr("عيد العمال"))

        # Independence Day.
        self._add_holiday_nov_15(tr("عيد الإستقلال"))

        # Catholic Christmas Day.
        self._add_christmas_day(tr("عيد الميلاد المجيد الغربي"), GREGORIAN_CALENDAR)

        # Hijri New Year.
        self._add_islamic_new_year_day(tr("رأس السنة الهجرية"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("ذكرى المولد النبوي الشريف"))

        # Isra' and Mi'raj.
        self._add_isra_and_miraj_day(tr("ذكرى الإسراء والمعراج"))

        # Eid al-Fitr.
        name = tr("عيد الفطر السعيد")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)

        # Eid al-Adha.
        name = tr("عيد الأضحى المبارك")
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)
        self._add_eid_al_adha_day_four(name)

    def _populate_catholic_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("رأس السنة الميلادي"))

        # Epiphany.
        self._add_epiphany_day(tr("عيد الغطاس"), GREGORIAN_CALENDAR)

        # Palm Sunday.
        self._add_palm_sunday(tr("أحد الشعانين"), GREGORIAN_CALENDAR)

        # Holy Thursday.
        self._add_holy_thursday(tr("خميس الغسل"), GREGORIAN_CALENDAR)

        # Good Friday.
        self._add_good_friday(tr("الجمعة العظيمة"), GREGORIAN_CALENDAR)

        # Holy Saturday.
        self._add_holy_saturday(tr("سبت النور"), GREGORIAN_CALENDAR)

        # Easter.
        self._add_easter_monday(tr("عيد الفصح المجيد"), GREGORIAN_CALENDAR)

        # Ascension Day.
        self._add_ascension_thursday(tr("خميس الصعود"), GREGORIAN_CALENDAR)

        # Pentecost.
        self._add_whit_sunday(tr("أحد العنصرة"), GREGORIAN_CALENDAR)

        # Catholic Christmas Day.
        self._add_christmas_day_two(tr("عيد الميلاد المجيد الغربي"), GREGORIAN_CALENDAR)

    def _populate_orthodox_holidays(self):
        # Orthodox New Year's Day.
        self._add_holiday_jan_14(tr("عيد رأس السنة الشرقي"))

        # Orthodox Christmas Day.
        self._add_christmas_day_two(tr("عيد الميلاد المجيد الشرقي"))

        # Epiphany.
        self._add_epiphany_day(tr("عيد الغطاس"))

        # Palm Sunday.
        self._add_palm_sunday(tr("أحد الشعانين"))

        # Holy Thursday.
        self._add_holy_thursday(tr("خميس الغسل"))

        # Good Friday.
        self._add_good_friday(tr("الجمعة العظيمة"))

        # Holy Saturday.
        self._add_holy_saturday(tr("سبت النور"))

        # Easter.
        self._add_easter_monday(tr("عيد الفصح المجيد"))

        # Ascension Day.
        self._add_ascension_thursday(tr("خميس الصعود"))

        # Pentecost.
        self._add_whit_sunday(tr("أحد العنصرة"))


class PS(Palestine):
    pass


class PSE(Palestine):
    pass


class PalestineIslamicHolidays(_CustomIslamicHolidays):
    MAWLID_DATES_CONFIRMED_YEARS = (2020, 2024)
